==> test.tree.jenkins-0.2.0-20121114.192319-6.jar: test/tree/debug.clj <==
(ns test.tree.debug
  (:require test.tree
            test.tree.reporter
            [fn.trace :as trace]
            [clojure.zip :as zip]))

(def test-traces (ref {}))

(defn add-trace
  "Add trace calls and results into a zipper tree structure, so that
   it can be printed out at any time."
  [trace-zip [entry out?]]
  (if out? 
    (-> trace-zip (zip/append-child entry) zip/up ) 
    (-> trace-zip (zip/append-child [entry]) zip/down zip/rightmost))) 

(defn wrap-tracing
  "If the suite is run with tracing on, save the trace in the results."
  [runner]
  (fn [{:keys [test] :as req}]
    (binding [trace/tracer (fn [_ value & [out?]]
                             (dosync
                              (alter test-traces update-in [test]
                                     (fn [trace-zip]
                                       (add-trace (or trace-zip (zip/vector-zip []))
                                                  [value out?])))))]
      (let [result (runner req)]
        (if (-> (:result result) (= :fail))
          (assoc-in result [:error :trace] (when-let [t (@test-traces test)]
                                             (zip/root t)))
          result)))))

(defmacro wrap-swank-conn-maybe
  "Produce a wrap-swank function that does nothing, if swank is not
   available."
  []
  (let [runnersym (gensym "runner")]
    `(defn wrap-swank
       "Allows you to place (swank.core/break) statements anywhere, and the
  swank debugger will stop there, no matter which thread hits that
  line."
       [~runnersym]
       ~(if (resolve 'swank.core.connection/*current-connection*)
          `(fn [req#]
             (let [conn# swank.core.connection/*current-connection*]
               (binding [swank.core.connection/*current-connection* conn#]
                 (~runnersym req#))))
          `(fn [req#]
             (~runnersym req#))))))

(wrap-swank-conn-maybe)

(defn debug
  "Run the given test tree, tracing all the functions in trace-list.
   The trace will be stored along with the rest of the test results.
   Accepts an optional reference for referring to the test results
   when running asynchronously.
   See also test.tree/run"
  [tree trace-list & [results-ref]]
  (with-redefs [test.tree/runner (-> test.tree/execute
                                    wrap-tracing
                                    test.tree/wrap-blockers
                                    test.tree/wrap-timer
                                    test.tree/wrap-data-driven
                                    wrap-swank)]
    (trace/dotrace trace-list
      (let [results (test.tree/run tree)]
        (when results-ref (reset! results-ref results))
        (doall (->> results second deref vals (map (comp deref :promise))))
        results))))

(defn quick-report
  "Print a quick summary of test results given result
  data returned by test.tree/run."
  [result]
  (let [reports (-> result second)]
    (binding [test.tree.reporter/*reports* reports]
      (let [fails (into {}
                        (filter (fn [[k v]] (test.tree.reporter/failed? k)) @reports))]
        {:failed-tests fails
         :counts {:failed (count fails)
                  :passed (count (test.tree.reporter/passed-tests))
                  :skipped (count (test.tree.reporter/skipped-tests))}}))))

==> test.tree.jenkins-0.2.0-20121128.195120-7.jar: test/tree/debug.clj <==
(ns test.tree.debug
  (:require test.tree
            test.tree.reporter
            [fn.trace :as trace]
            [clojure.zip :as zip]))

(def test-traces (ref {}))

(defn add-trace
  "Add trace calls and results into a zipper tree structure, so that
   it can be printed out at any time."
  [trace-zip [entry out?]]
  (if out? 
    (-> trace-zip (zip/append-child entry) zip/up ) 
    (-> trace-zip (zip/append-child [entry]) zip/down zip/rightmost))) 

(defn tree-trace "Turn a list of traces into a pretty printable tree"
  [trace-list]
  (->> trace-list
       (reduce add-trace (zip/vector-zip []))
       zip/root))

(deftype TestTrace [trace-list])

(defmethod clojure.pprint/simple-dispatch TestTrace [o]
  (println "TestTrace simple-dispatch")
  (-> o .trace-list tree-trace clojure.pprint/simple-dispatch))

(defmethod print-method TestTrace [o w]
  (pr (.trace-list o)))

(defn wrap-tracing
  "If the suite is run with tracing on, save the trace in the results."
  [runner]
  (fn [{:keys [test] :as req}]
    (binding [trace/tracer (fn [_ value & [out?]]
                             (dosync
                              (alter test-traces update-in [test]
                                     (fn [trace-list]
                                       (let [entry (vector value out?)]
                                         (if trace-list
                                                 (conj trace-list entry)
                                                 (vector entry)))))))]
      (let [result (runner req)]
        (assoc-in result [:error :trace]
                  (TestTrace. (@test-traces test)))))))

(defmacro wrap-swank-conn-maybe
  "Produce a wrap-swank function that does nothing, if swank is not
   available."
  []
  (let [runnersym (gensym "runner")]
    `(defn wrap-swank
       "Allows you to place (swank.core/break) statements anywhere, and the
  swank debugger will stop there, no matter which thread hits that
  line."
       [~runnersym]
       ~(if (resolve 'swank.core.connection/*current-connection*)
          `(fn [req#]
             (let [conn# swank.core.connection/*current-connection*]
               (binding [swank.core.connection/*current-connection* conn#]
                 (~runnersym req#))))
          `(fn [req#]
             (~runnersym req#))))))

(wrap-swank-conn-maybe)

(defn debug
  "Run the given test tree, tracing all the functions in trace-list.
   The trace will be stored along with the rest of the test results.
   Accepts an optional reference for referring to the test results
   when running asynchronously.
   See also test.tree/run"
  [tree trace-list & [results-ref]]
  (with-redefs [test.tree/runner (-> test.tree/execute
                                    wrap-tracing
                                    test.tree/wrap-blockers
                                    test.tree/wrap-timer
                                    test.tree/wrap-data-driven
                                    wrap-swank)]
    (trace/dotrace trace-list
      (let [results (test.tree/run tree)]
        (when results-ref (reset! results-ref results))
        (doall (->> results second deref vals (map (comp deref :promise))))
        results))))

(defn quick-report
  "Print a quick summary of test results given result
  data returned by test.tree/run."
  [result]
  (let [reports (-> result second)]
    (binding [test.tree.reporter/*reports* reports]
      (let [fails (into {}
                        (filter (fn [[k v]] (test.tree.reporter/failed? k)) @reports))]
        {:failed-tests fails
         :counts {:failed (count fails)
                  :passed (count (test.tree.reporter/passed-tests))
                  :skipped (count (test.tree.reporter/skipped-tests))}}))))
